@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -127,7 +127,9 @@ tracker_data_backup_restore (GFile *journal,
                              TrackerDataBackupFinished callback,
                              gpointer user_data,
                              GDestroyNotify destroy,
-                             const gchar **test_schemas)
+                             const gchar **test_schemas,
+                             TrackerBusyCallback busy_callback,
+                             gpointer busy_user_data)
 {
 	BackupSaveInfo *info;
 
@@ -164,7 +166,9 @@ tracker_data_backup_restore (GFile *journal,
 
 		tracker_db_journal_shutdown ();
 
-		tracker_data_manager_init (flags, test_schemas, &is_first, TRUE);
+		tracker_data_manager_init (flags, test_schemas, &is_first, TRUE,
+		                           busy_callback, busy_user_data,
+		                           "Restoring backup");
 
 	} else {
 		g_set_error (&info->error, TRACKER_DATA_BACKUP_ERROR, 0, 
