@@ -310,6 +310,21 @@ tracker_db_interface_execute_query (TrackerDBInterface  *interface,
 	return result_set;
 }
 
+gboolean
+tracker_db_interface_interrupt (TrackerDBInterface *interface)
+{
+	g_return_val_if_fail (TRACKER_IS_DB_INTERFACE (interface), FALSE);
+
+	if (!TRACKER_DB_INTERFACE_GET_IFACE (interface)->interrupt) {
+		g_critical ("Database abstraction %s doesn't implement "
+		            "the method interrupt()",
+		            G_OBJECT_TYPE_NAME (interface));
+		return FALSE;
+	}
+
+	return TRACKER_DB_INTERFACE_GET_IFACE (interface)->interrupt (interface);
+}
+
 gboolean
 tracker_db_interface_start_transaction (TrackerDBInterface *interface)
 {
@@ -461,14 +476,15 @@ tracker_db_cursor_rewind (TrackerDBCursor *cursor)
 }
 
 gboolean
-tracker_db_cursor_iter_next (TrackerDBCursor *cursor)
+tracker_db_cursor_iter_next (TrackerDBCursor *cursor,
+                             GError         **error)
 {
 	/* Removed for performance 
 	g_return_val_if_fail (TRACKER_IS_DB_CURSOR (cursor), FALSE); */
 
 	g_return_val_if_fail (cursor != NULL, FALSE);
 
-	return TRACKER_DB_CURSOR_GET_IFACE (cursor)->iter_next (cursor);
+	return TRACKER_DB_CURSOR_GET_IFACE (cursor)->iter_next (cursor, error);
 }
 
 guint
