@@ -45,6 +45,7 @@
 #define DEFAULT_THROTTLE                         0        /* 0->20 */
 #define DEFAULT_SCAN_TIMEOUT                     0        /* 0->1000 */
 #define DEFAULT_CACHE_TIMEOUT                    60       /* 0->1000 */
+#define DEFAULT_INDEX_MOUNTED_DIRECTORIES        TRUE
 #define DEFAULT_INDEX_REMOVABLE_DEVICES          TRUE
 #define DEFAULT_INDEX_OPTICAL_DISCS              FALSE
 #define DEFAULT_INDEX_ON_BATTERY                 FALSE
@@ -65,6 +66,7 @@ typedef struct {
 	gint      throttle;
 	gboolean  index_on_battery;
 	gboolean  index_on_battery_first_time;
+	gboolean  index_mounted_directories;
 	gboolean  index_removable_devices;
 	gboolean  index_optical_discs;
 	gint      low_disk_space_limit;
@@ -124,6 +126,7 @@ enum {
 	PROP_THROTTLE,
 	PROP_INDEX_ON_BATTERY,
 	PROP_INDEX_ON_BATTERY_FIRST_TIME,
+	PROP_INDEX_MOUNTED_DIRECTORIES,
 	PROP_INDEX_REMOVABLE_DEVICES,
 	PROP_INDEX_OPTICAL_DISCS,
 	PROP_LOW_DISK_SPACE_LIMIT,
@@ -146,6 +149,7 @@ static ObjectToKeyFile conversions[] = {
 	{ G_TYPE_INT,     "throttle",                         GROUP_INDEXING, "Throttle"                  },
 	{ G_TYPE_BOOLEAN, "index-on-battery",                 GROUP_INDEXING, "IndexOnBattery"            },
 	{ G_TYPE_BOOLEAN, "index-on-battery-first-time",      GROUP_INDEXING, "IndexOnBatteryFirstTime"   },
+	{ G_TYPE_BOOLEAN, "index-mounted-directories",        GROUP_INDEXING, "IndexMountedDirectories"   },
 	{ G_TYPE_BOOLEAN, "index-removable-devices",          GROUP_INDEXING, "IndexRemovableMedia"       },
 	{ G_TYPE_BOOLEAN, "index-optical-discs",              GROUP_INDEXING, "IndexOpticalDiscs"         },
 	{ G_TYPE_INT,     "low-disk-space-limit",             GROUP_INDEXING, "LowDiskSpaceLimit"         },
@@ -242,6 +246,14 @@ tracker_config_class_init (TrackerConfigClass *klass)
 	                                                       " Set to true to index while running on battery for the first time only",
 	                                                       DEFAULT_INDEX_ON_BATTERY_FIRST_TIME,
 	                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+	g_object_class_install_property (object_class,
+	                                 PROP_INDEX_MOUNTED_DIRECTORIES,
+	                                 g_param_spec_boolean ("index-mounted-directories",
+	                                                       "Index mounted directories",
+	                                                       " Set to true to enable traversing mounted directories on other file systems\n"
+	                                                       " (this excludes removable devices)",
+	                                                       DEFAULT_INDEX_MOUNTED_DIRECTORIES,
+	                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 	g_object_class_install_property (object_class,
 	                                 PROP_INDEX_REMOVABLE_DEVICES,
 	                                 g_param_spec_boolean ("index-removable-devices",
@@ -362,6 +374,10 @@ config_set_property (GObject      *object,
 		tracker_config_set_index_on_battery_first_time (TRACKER_CONFIG (object),
 		                                                g_value_get_boolean (value));
 		break;
+	case PROP_INDEX_MOUNTED_DIRECTORIES:
+		tracker_config_set_index_mounted_directories (TRACKER_CONFIG (object),
+		                                              g_value_get_boolean (value));
+		break;
 	case PROP_INDEX_REMOVABLE_DEVICES:
 		tracker_config_set_index_removable_devices (TRACKER_CONFIG (object),
 		                                            g_value_get_boolean (value));
@@ -441,6 +457,9 @@ config_get_property (GObject    *object,
 	case PROP_INDEX_ON_BATTERY_FIRST_TIME:
 		g_value_set_boolean (value, priv->index_on_battery_first_time);
 		break;
+	case PROP_INDEX_MOUNTED_DIRECTORIES:
+		g_value_set_boolean (value, priv->index_mounted_directories);
+		break;
 	case PROP_INDEX_REMOVABLE_DEVICES:
 		g_value_set_boolean (value, priv->index_removable_devices);
 		break;
@@ -833,6 +852,7 @@ config_load (TrackerConfig *config,
 
 	for (i = 0; i < G_N_ELEMENTS (conversions); i++) {
 		gboolean has_key;
+		gboolean is_directory_list;
 
 		has_key = g_key_file_has_key (file->key_file,
 		                              conversions[i].group,
@@ -858,69 +878,72 @@ config_load (TrackerConfig *config,
 
 		case G_TYPE_POINTER: {
 			GSList *new_dirs, *old_dirs, *l;
-			gboolean is_recursive = TRUE;
-			gboolean equal;
+			gboolean check_for_duplicates = FALSE;
+                        gboolean is_recursive = TRUE;
+                        gboolean equal;
 
-			if (strcmp (conversions[i].property, "index-recursive-directories") == 0 ||
-			    strcmp (conversions[i].property, "index-single-directories") == 0 ||
-			    strcmp (conversions[i].property, "ignored-directories") == 0) {
-				is_recursive = strcmp (conversions[i].property, "index-recursive-directories") == 0;
-				tracker_keyfile_object_load_directory_list (G_OBJECT (file),
-				                                            conversions[i].property,
-				                                            file->key_file,
-				                                            conversions[i].group,
-				                                            conversions[i].key,
-				                                            is_recursive,
-				                                            &new_dirs);
+			is_directory_list = TRUE;
 
-				for (l = new_dirs; l; l = l->next) {
-					const gchar *path_to_use;
-
-					/* Must be a special dir */
-					if (strcmp (l->data, "&DESKTOP") == 0) {
-						path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP);
-					} else if (strcmp (l->data, "&DOCUMENTS") == 0) {
-						path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_DOCUMENTS);
-					} else if (strcmp (l->data, "&DOWNLOAD") == 0) {
-						path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_DOWNLOAD);
-					} else if (strcmp (l->data, "&MUSIC") == 0) {
-						path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_MUSIC);
-					} else if (strcmp (l->data, "&PICTURES") == 0) {
-						path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
-					} else if (strcmp (l->data, "&PUBLIC_SHARE") == 0) {
-						path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_PUBLIC_SHARE);
-					} else if (strcmp (l->data, "&TEMPLATES") == 0) {
-						path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_TEMPLATES);
-					} else if (strcmp (l->data, "&VIDEOS") == 0) {
-						path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_VIDEOS);
-					} else {
-						path_to_use = NULL;
-					}
-
-					if (path_to_use) {
-						g_free (l->data);
-						l->data = g_strdup (path_to_use);
-					}
-				}
-			} else {
-				tracker_keyfile_object_load_string_list (G_OBJECT (file),
-				                                         conversions[i].property,
-				                                         file->key_file,
-				                                         conversions[i].group,
-				                                         conversions[i].key,
-				                                         &new_dirs);
+			if (strcmp (conversions[i].property, "index-recursive-directories") != 0 &&
+			    strcmp (conversions[i].property, "index-single-directories") != 0) {
+                                tracker_keyfile_object_load_string_list (G_OBJECT (file),
+                                                                         conversions[i].property,
+                                                                         file->key_file,
+                                                                         conversions[i].group,
+                                                                         conversions[i].key,
+                                                                         NULL);
+                                continue;
 			}
 
-			g_object_get (config, conversions[i].property, &old_dirs, NULL);
+                        is_recursive = strcmp (conversions[i].property, "index-recursive-directories") == 0;
+                        tracker_keyfile_object_load_directory_list (G_OBJECT (file),
+                                                                    conversions[i].property,
+                                                                    file->key_file,
+                                                                    conversions[i].group,
+                                                                    conversions[i].key,
+                                                                    is_recursive,
+                                                                    &new_dirs);
+                        g_object_get (config, conversions[i].property, &old_dirs, NULL);
 
-			equal = tracker_gslist_with_string_data_equal (new_dirs, old_dirs);
+			for (l = new_dirs; l; l = l->next) {
+				const gchar *path_to_use;
 
-			if (!equal) {
-				g_object_set (config, conversions[i].property, new_dirs, NULL);
+				/* Must be a special dir */
+				if (strcmp (l->data, "&DESKTOP") == 0) {
+					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP);
+				} else if (strcmp (l->data, "&DOCUMENTS") == 0) {
+					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_DOCUMENTS);
+				} else if (strcmp (l->data, "&DOWNLOAD") == 0) {
+					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_DOWNLOAD);
+				} else if (strcmp (l->data, "&MUSIC") == 0) {
+					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_MUSIC);
+				} else if (strcmp (l->data, "&PICTURES") == 0) {
+					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
+				} else if (strcmp (l->data, "&PUBLIC_SHARE") == 0) {
+					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_PUBLIC_SHARE);
+				} else if (strcmp (l->data, "&TEMPLATES") == 0) {
+					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_TEMPLATES);
+				} else if (strcmp (l->data, "&VIDEOS") == 0) {
+					path_to_use = g_get_user_special_dir (G_USER_DIRECTORY_VIDEOS);
+				} else {
+					path_to_use = NULL;
+				}
+
+				if (path_to_use) {
+					check_for_duplicates = TRUE;
+					g_free (l->data);
+					l->data = g_strdup (path_to_use);
+				}
+			}
+
+                        equal = tracker_gslist_with_string_data_equal (new_dirs, old_dirs);
+
+                        if (!equal) {
+                                g_object_set (config, conversions[i].property, new_dirs, NULL);
                         }
 
-			g_slist_foreach (new_dirs, (GFunc) g_free, NULL);
-			g_slist_free (new_dirs);
+                        g_slist_foreach (new_dirs, (GFunc) g_free, NULL);
+                        g_slist_free (new_dirs);
 
 			break;
 		}
@@ -982,12 +1005,7 @@ config_save (TrackerConfig *config)
 					 * breaks. Need a better
 					 * solution at some point.
 					 */
-					if (g_strcmp0 (l->data, g_get_home_dir ()) == 0) {
-						/* Home dir gets straight into configuration,
-						 * regardless of having XDG dirs pointing to it.
-						 */
-						path_to_use = "$HOME";
-					} else if (g_strcmp0 (l->data, g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP)) == 0) {
+					if (g_strcmp0 (l->data, g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP)) == 0) {
 						path_to_use = "&DESKTOP";
 					} else if (g_strcmp0 (l->data, g_get_user_special_dir (G_USER_DIRECTORY_DOCUMENTS)) == 0) {
 						path_to_use = "&DOCUMENTS";
@@ -1147,6 +1165,18 @@ tracker_config_get_index_on_battery_first_time (TrackerConfig *config)
 	return priv->index_on_battery_first_time;
 }
 
+gboolean
+tracker_config_get_index_mounted_directories (TrackerConfig *config)
+{
+	TrackerConfigPrivate *priv;
+
+	g_return_val_if_fail (TRACKER_IS_CONFIG (config), DEFAULT_INDEX_MOUNTED_DIRECTORIES);
+
+	priv = TRACKER_CONFIG_GET_PRIVATE (config);
+
+	return priv->index_mounted_directories;
+}
+
 gboolean
 tracker_config_get_index_removable_devices (TrackerConfig *config)
 {
@@ -1399,6 +1429,20 @@ tracker_config_set_index_on_battery_first_time (TrackerConfig *config,
 	g_object_notify (G_OBJECT (config), "index-on-battery-first-time");
 }
 
+void
+tracker_config_set_index_mounted_directories (TrackerConfig *config,
+                                              gboolean       value)
+{
+	TrackerConfigPrivate *priv;
+
+	g_return_if_fail (TRACKER_IS_CONFIG (config));
+
+	priv = TRACKER_CONFIG_GET_PRIVATE (config);
+
+	priv->index_mounted_directories = value;
+	g_object_notify (G_OBJECT (config), "index-mounted-directories");
+}
+
 void
 tracker_config_set_index_removable_devices (TrackerConfig *config,
                                             gboolean       value)
